@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2010-06-04 04:34:20 +1000 (Fri, 04 Jun 2010) $ by $Author: schulte $
- *     $Revision: 11017 $
+ *     $Date: 2012-09-03 17:08:20 +0200 (Mon, 03 Sep 2012) $ by $Author: schulte $
+ *     $Revision: 13038 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -39,10 +39,9 @@
 
 namespace Gecode {
 
-  using namespace Int;
-
   void
   abs(Home home, IntVar x0, IntVar x1, IntConLevel icl) {
+    using namespace Int;
     if (home.failed()) return;
     if (icl == ICL_DOM) {
       GECODE_ES_FAIL(Arithmetic::AbsDom<IntView>::post(home,x0,x1));
@@ -55,6 +54,7 @@ namespace Gecode {
   void
   max(Home home, IntVar x0, IntVar x1, IntVar x2,
       IntConLevel icl) {
+    using namespace Int;
     if (home.failed()) return;
     if (icl == ICL_DOM) {
       GECODE_ES_FAIL(Arithmetic::MaxDom<IntView>::post(home,x0,x1,x2));
@@ -66,6 +66,7 @@ namespace Gecode {
   void
   max(Home home, const IntVarArgs& x, IntVar y,
       IntConLevel icl) {
+    using namespace Int;
     if (x.size() == 0)
       throw TooFewArguments("Int::max");
     if (home.failed()) return;
@@ -77,10 +78,10 @@ namespace Gecode {
     }
   }
 
-
   void
   min(Home home, IntVar x0, IntVar x1, IntVar x2,
       IntConLevel icl) {
+    using namespace Int;
     if (home.failed()) return;
     MinusView m0(x0); MinusView m1(x1); MinusView m2(x2);
     if (icl == ICL_DOM) {
@@ -93,6 +94,7 @@ namespace Gecode {
   void
   min(Home home, const IntVarArgs& x, IntVar y,
       IntConLevel icl) {
+    using namespace Int;
     if (x.size() == 0)
       throw TooFewArguments("Int::min");
     if (home.failed()) return;
@@ -111,51 +113,32 @@ namespace Gecode {
   void
   mult(Home home, IntVar x0, IntVar x1, IntVar x2,
        IntConLevel icl) {
+    using namespace Int;
     if (home.failed()) return;
     if (icl == ICL_DOM) {
-      GECODE_ES_FAIL(Arithmetic::MultDom<IntView>::post(home,x0,x1,x2));
+      GECODE_ES_FAIL(Arithmetic::MultDom::post(home,x0,x1,x2));
     } else {
-      GECODE_ES_FAIL(Arithmetic::MultBnd<IntView>::post(home,x0,x1,x2));
+      GECODE_ES_FAIL(Arithmetic::MultBnd::post(home,x0,x1,x2));
     }
   }
 
 
-  void
-  sqr(Home home, IntVar x0, IntVar x1, IntConLevel icl) {
-    if (home.failed()) return;
-    if (icl == ICL_DOM) {
-      GECODE_ES_FAIL(Arithmetic::SqrDom<IntView>::post(home,x0,x1));
-    } else {
-      GECODE_ES_FAIL(Arithmetic::SqrBnd<IntView>::post(home,x0,x1));
-    }
-  }
-
-  void
-  sqrt(Home home, IntVar x0, IntVar x1, IntConLevel icl) {
-    if (home.failed()) return;
-    if (icl == ICL_DOM) {
-      GECODE_ES_FAIL(Arithmetic::SqrtDom<IntView>::post(home,x0,x1));
-    } else {
-      GECODE_ES_FAIL(Arithmetic::SqrtBnd<IntView>::post(home,x0,x1));
-    }
-  }
-
   void
   divmod(Home home, IntVar x0, IntVar x1, IntVar x2, IntVar x3,
          IntConLevel) {
+    using namespace Int;
     if (home.failed()) return;
 
     IntVar prod(home, Int::Limits::min, Int::Limits::max);
-    GECODE_ES_FAIL(
-                   Arithmetic::MultBnd<IntView>::post(home,x1,x2,prod));
+    GECODE_ES_FAIL(Arithmetic::MultBnd::post(home,x1,x2,prod));
     Linear::Term<IntView> t[3];
     t[0].a = 1; t[0].x = prod;
     t[1].a = 1; t[1].x = x3;
     int min, max;
     Linear::estimate(t,2,0,min,max);
     IntView x0v(x0);
-    GECODE_ME_FAIL( x0v.gq(home,min));
-    GECODE_ME_FAIL( x0v.lq(home,max));
+    GECODE_ME_FAIL(x0v.gq(home,min));
+    GECODE_ME_FAIL(x0v.lq(home,max));
     t[2].a=-1; t[2].x=x0;
     Linear::post(home,t,3,IRT_EQ,0);
     if (home.failed()) return;
@@ -167,6 +150,7 @@ namespace Gecode {
   void
   div(Home home, IntVar x0, IntVar x1, IntVar x2,
       IntConLevel) {
+    using namespace Int;
     if (home.failed()) return;
     GECODE_ES_FAIL(
       (Arithmetic::DivBnd<IntView>::post(home,x0,x1,x2)));
@@ -175,11 +159,78 @@ namespace Gecode {
   void
   mod(Home home, IntVar x0, IntVar x1, IntVar x2,
       IntConLevel icl) {
+    using namespace Int;
     if (home.failed()) return;
     IntVar _div(home, Int::Limits::min, Int::Limits::max);
     divmod(home, x0, x1, _div, x2, icl);
   }
 
+  void
+  sqr(Home home, IntVar x0, IntVar x1, IntConLevel icl) {
+    using namespace Int;
+    if (home.failed()) return;
+    Arithmetic::SqrOps ops;
+    if (icl == ICL_DOM) {
+      GECODE_ES_FAIL(Arithmetic::PowDom<Arithmetic::SqrOps>
+                     ::post(home,x0,x1,ops));
+    } else {
+      GECODE_ES_FAIL(Arithmetic::PowBnd<Arithmetic::SqrOps>
+                     ::post(home,x0,x1,ops));
+    }
+  }
+
+  void
+  sqrt(Home home, IntVar x0, IntVar x1, IntConLevel icl) {
+    using namespace Int;
+    if (home.failed()) return;
+    Arithmetic::SqrOps ops;
+    if (icl == ICL_DOM) {
+      GECODE_ES_FAIL(Arithmetic::NrootDom<Arithmetic::SqrOps>
+                     ::post(home,x0,x1,ops));
+    } else {
+      GECODE_ES_FAIL(Arithmetic::NrootBnd<Arithmetic::SqrOps>
+                     ::post(home,x0,x1,ops));
+    }
+  }
+
+  void
+  pow(Home home, IntVar x0, int n, IntVar x1, IntConLevel icl) {
+    using namespace Int;
+    Limits::nonnegative(n,"Int::pow");
+    if (home.failed()) return;
+    if (n == 2) {
+      sqr(home, x0, x1, icl);
+      return;
+    }
+    Arithmetic::PowOps ops(n);
+    if (icl == ICL_DOM) {
+      GECODE_ES_FAIL(Arithmetic::PowDom<Arithmetic::PowOps>
+                     ::post(home,x0,x1,ops));
+    } else {
+      GECODE_ES_FAIL(Arithmetic::PowBnd<Arithmetic::PowOps>
+                     ::post(home,x0,x1,ops));
+    }
+  }
+
+  void
+  nroot(Home home, IntVar x0, int n, IntVar x1, IntConLevel icl) {
+    using namespace Int;
+    Limits::positive(n,"Int::nroot");
+    if (home.failed()) return;
+    if (n == 2) {
+      sqrt(home, x0, x1, icl);
+      return;
+    }
+    Arithmetic::PowOps ops(n);
+    if (icl == ICL_DOM) {
+      GECODE_ES_FAIL(Arithmetic::NrootDom<Arithmetic::PowOps>
+                     ::post(home,x0,x1,ops));
+    } else {
+      GECODE_ES_FAIL(Arithmetic::NrootBnd<Arithmetic::PowOps>
+                     ::post(home,x0,x1,ops));
+    }
+  }
+
 }
 
 // STATISTICS: int-post
