@@ -7,8 +7,8 @@
  *     Christian Schulte, 2002
  *
  *  Last modified:
- *     $Date: 2011-08-12 23:27:20 +1000 (Fri, 12 Aug 2011) $ by $Author: tack $
- *     $Revision: 12285 $
+ *     $Date: 2013-02-14 16:29:11 +0100 (Thu, 14 Feb 2013) $ by $Author: schulte $
+ *     $Revision: 13292 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -43,10 +43,10 @@ namespace Gecode { namespace Int { namespace Linear {
   template<class View>
   inline void
   estimate(Term<View>* t, int n, int c, int& l, int &u) {
-    double min = c;
-    double max = c;
+    long long int min = c;
+    long long int max = c;
     for (int i=n; i--; ) {
-      double a = t[i].a;
+      long long int a = t[i].a;
       if (a > 0) {
         min += a*t[i].x.min();
         max += a*t[i].x.max();
@@ -77,11 +77,49 @@ namespace Gecode { namespace Int { namespace Linear {
     }
   };
 
+  /// Compute the greatest common divisor of \a a and \a b
+  inline int 
+  gcd(int a, int b) {
+    if (b > a) 
+      std::swap(a,b);
+    while (b > 0) {
+      int t = b; b = a % b; a = t;
+    }
+    return a;
+  }
+
+
+
+  /** \brief Normalize linear integer constraints
+   *
+   * \param t array of linear terms
+   * \param n size of array
+   * \param t_p array of linear terms over integers with positive coefficients
+   * \param n_p number of postive terms
+   * \param t_n array of linear terms over integers with negative coefficients
+   * \param n_n number of negative terms
+   * \param gcd greatest common divisor of all coefficients
+   *
+   * Replaces all negative coefficients by positive coefficients.
+   *
+   *  - Variables occuring multiply in the term array are replaced
+   *    by a single occurence: for example, \f$ax+bx\f$ becomes
+   *    \f$(a+b)x\f$.
+   *  - If in the above simplification the value for \f$(a+b)\f$ (or for
+   *    \f$a\f$ and \f$b\f$) exceeds the limits for integers as
+   *    defined in Limits::Int, an exception of type
+   *    Int::NumericalOverflow is thrown.
+   *  - Divides all coefficients by their greatest common divisor and
+   *    returns the gcd \a g
+   *
+   * Returns true, if all coefficients are unit coefficients
+   */
   template<class View>
   inline bool
   normalize(Term<View>* t, int &n,
             Term<View>* &t_p, int &n_p,
-            Term<View>* &t_n, int &n_n) {
+            Term<View>* &t_n, int &n_n, 
+            int& g) {
     /*
      * Join coefficients for aliased variables:
      *
@@ -96,13 +134,13 @@ namespace Gecode { namespace Int { namespace Linear {
       int j = 0;
       while (i < n) {
         Limits::check(t[i].a,"Int::linear");
-        double a = t[i].a;
+        long long int a = t[i].a;
         View x = t[i].x;
         while ((++i < n) && same(t[i].x,x)) {
           a += t[i].a;
           Limits::check(a,"Int::linear");
         }
-        if (a != 0.0) {
+        if (a != 0) {
           t[j].a = static_cast<int>(a); t[j].x = x; j++;
         }
       }
@@ -136,6 +174,21 @@ namespace Gecode { namespace Int { namespace Linear {
     for (int i=n_n; i--; )
       t_n[i].a = -t_n[i].a;
 
+    /*
+     * Compute greatest common divisor
+     *
+     */
+    if ((n > 0) && (g > 0)) {
+      g = t[0].a;
+      for (int i=1; (g > 1) && (i < n); i++)
+        g = gcd(t[i].a,g);
+      if (g > 1)
+        for (int i=n; i--; )
+          t[i].a /= g;
+    } else {
+      g = 1;
+    }
+
     /*
      * Test for unit coefficients only
      *
