@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2011-08-09 02:04:53 +1000 (Tue, 09 Aug 2011) $ by $Author: schulte $
- *     $Revision: 12253 $
+ *     $Date: 2012-10-18 16:02:42 +0200 (Thu, 18 Oct 2012) $ by $Author: schulte $
+ *     $Revision: 13154 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -428,55 +428,71 @@ namespace Gecode { namespace Int { namespace Rel {
    *
    */
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqDom<View,CtrlView>::ReEqDom(Home home, View x0, View x1, CtrlView b)
+  ReEqDom<View,CtrlView,rm>::ReEqDom(Home home, View x0, View x1, CtrlView b)
     : ReBinaryPropagator<View,PC_INT_DOM,CtrlView>(home,x0,x1,b) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqDom<View,CtrlView>::post(Home home, View x0, View x1, CtrlView b) {
-    if (b.one())
+  ReEqDom<View,CtrlView,rm>::post(Home home, View x0, View x1, CtrlView b) {
+    if (b.one()) {
+      if (rm == RM_PMI)
+        return ES_OK;
       return EqDom<View,View>::post(home,x0,x1);
-    if (b.zero())
+    }
+    if (b.zero()) {
+      if (rm == RM_IMP)
+        return ES_OK;
       return Nq<View>::post(home,x0,x1);
+    }
     if (!same(x0,x1)) {
       (void) new (home) ReEqDom(home,x0,x1,b);
-    } else {
+    } else if (rm != RM_IMP) {
       GECODE_ME_CHECK(b.one(home));
     }
     return ES_OK;
   }
 
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqDom<View,CtrlView>::ReEqDom(Space& home, bool share, ReEqDom& p)
+  ReEqDom<View,CtrlView,rm>::ReEqDom(Space& home, bool share, ReEqDom& p)
     : ReBinaryPropagator<View,PC_INT_DOM,CtrlView>(home,share,p) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   Actor*
-  ReEqDom<View,CtrlView>::copy(Space& home, bool share) {
-    return new (home) ReEqDom<View,CtrlView>(home,share,*this);
+  ReEqDom<View,CtrlView,rm>::copy(Space& home, bool share) {
+    return new (home) ReEqDom<View,CtrlView,rm>(home,share,*this);
   }
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqDom<View,CtrlView>::propagate(Space& home, const ModEventDelta&) {
-    if (b.one())
+  ReEqDom<View,CtrlView,rm>::propagate(Space& home, const ModEventDelta&) {
+    if (b.one()) {
+      if (rm == RM_PMI)
+        return home.ES_SUBSUMED(*this);
       GECODE_REWRITE(*this,(EqDom<View,View>::post(home(*this),x0,x1)));
-    if (b.zero())
+    }
+    if (b.zero()) {
+      if (rm == RM_IMP)
+        return home.ES_SUBSUMED(*this);        
       GECODE_REWRITE(*this,Nq<View>::post(home(*this),x0,x1));
+    }
     switch (rtest_eq_dom(x0,x1)) {
     case RT_TRUE:
-      GECODE_ME_CHECK(b.one_none(home)); return home.ES_SUBSUMED(*this);
-    case RT_FALSE:
-      GECODE_ME_CHECK(b.zero_none(home)); return home.ES_SUBSUMED(*this);
-    case RT_MAYBE:
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(b.one_none(home)); 
       break;
+    case RT_FALSE:
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(b.zero_none(home)); 
+      break;
+    case RT_MAYBE:
+      return ES_FIX;
     default: GECODE_NEVER;
     }
-    return ES_FIX;
+    return home.ES_SUBSUMED(*this);
   }
 
 
@@ -486,55 +502,71 @@ namespace Gecode { namespace Int { namespace Rel {
    *
    */
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqBnd<View,CtrlView>::ReEqBnd(Home home, View x0, View x1, CtrlView b)
+  ReEqBnd<View,CtrlView,rm>::ReEqBnd(Home home, View x0, View x1, CtrlView b)
     : ReBinaryPropagator<View,PC_INT_BND,CtrlView>(home,x0,x1,b) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqBnd<View,CtrlView>::post(Home home, View x0, View x1, CtrlView b){
-    if (b.one())
+  ReEqBnd<View,CtrlView,rm>::post(Home home, View x0, View x1, CtrlView b){
+    if (b.one()) {
+      if (rm == RM_PMI)
+        return ES_OK;
       return EqBnd<View,View>::post(home,x0,x1);
-    if (b.zero())
+    }
+    if (b.zero()) {
+      if (rm == RM_IMP)
+        return ES_OK;
       return Nq<View>::post(home,x0,x1);
+    }
     if (!same(x0,x1)) {
       (void) new (home) ReEqBnd(home,x0,x1,b);
-    } else {
+    } else if (rm != RM_IMP) {
       GECODE_ME_CHECK(b.one(home));
     }
     return ES_OK;
   }
 
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqBnd<View,CtrlView>::ReEqBnd(Space& home, bool share, ReEqBnd& p)
+  ReEqBnd<View,CtrlView,rm>::ReEqBnd(Space& home, bool share, ReEqBnd& p)
     : ReBinaryPropagator<View,PC_INT_BND,CtrlView>(home,share,p) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   Actor*
-  ReEqBnd<View,CtrlView>::copy(Space& home, bool share) {
-    return new (home) ReEqBnd<View,CtrlView>(home,share,*this);
+  ReEqBnd<View,CtrlView,rm>::copy(Space& home, bool share) {
+    return new (home) ReEqBnd<View,CtrlView,rm>(home,share,*this);
   }
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqBnd<View,CtrlView>::propagate(Space& home, const ModEventDelta&) {
-    if (b.one())
+  ReEqBnd<View,CtrlView,rm>::propagate(Space& home, const ModEventDelta&) {
+    if (b.one()) {
+      if (rm == RM_PMI)
+        return home.ES_SUBSUMED(*this);
       GECODE_REWRITE(*this,(EqBnd<View,View>::post(home(*this),x0,x1)));
-    if (b.zero())
+    }
+    if (b.zero()) {
+      if (rm == RM_IMP)
+        return home.ES_SUBSUMED(*this);
       GECODE_REWRITE(*this,Nq<View>::post(home(*this),x0,x1));
+    }
     switch (rtest_eq_bnd(x0,x1)) {
     case RT_TRUE:
-      GECODE_ME_CHECK(b.one_none(home));  return home.ES_SUBSUMED(*this);
-    case RT_FALSE:
-      GECODE_ME_CHECK(b.zero_none(home)); return home.ES_SUBSUMED(*this);
-    case RT_MAYBE:
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(b.one_none(home));
       break;
+    case RT_FALSE:
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(b.zero_none(home));
+      break;
+    case RT_MAYBE:
+      return ES_FIX;
     default: GECODE_NEVER;
     }
-    return ES_FIX;
+    return home.ES_SUBSUMED(*this);
   }
 
 
@@ -545,25 +577,29 @@ namespace Gecode { namespace Int { namespace Rel {
    *
    */
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqDomInt<View,CtrlView>::ReEqDomInt
+  ReEqDomInt<View,CtrlView,rm>::ReEqDomInt
   (Home home, View x, int c0, CtrlView b)
     : ReUnaryPropagator<View,PC_INT_DOM,CtrlView>(home,x,b), c(c0) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqDomInt<View,CtrlView>::post(Home home, View x, int c, CtrlView b) {
+  ReEqDomInt<View,CtrlView,rm>::post(Home home, View x, int c, CtrlView b) {
     if (b.one()) {
-      GECODE_ME_CHECK(x.eq(home,c));
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(x.eq(home,c));
     } else if (b.zero()) {
-      GECODE_ME_CHECK(x.nq(home,c));
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(x.nq(home,c));
     } else if (x.assigned()) {
       assert(b.none());
       if (x.val() == c) {
-        GECODE_ME_CHECK(b.one_none(home));
+        if (rm != RM_IMP)
+          GECODE_ME_CHECK(b.one_none(home));
       } else {
-        GECODE_ME_CHECK(b.zero_none(home));
+        if (rm != RM_PMI)
+          GECODE_ME_CHECK(b.zero_none(home));
       }
     } else {
       (void) new (home) ReEqDomInt(home,x,c,b);
@@ -572,30 +608,37 @@ namespace Gecode { namespace Int { namespace Rel {
   }
 
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqDomInt<View,CtrlView>::ReEqDomInt(Space& home, bool share, ReEqDomInt& p)
+  ReEqDomInt<View,CtrlView,rm>::ReEqDomInt(Space& home, bool share,
+                                           ReEqDomInt& p)
     : ReUnaryPropagator<View,PC_INT_DOM,CtrlView>(home,share,p), c(p.c) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   Actor*
-  ReEqDomInt<View,CtrlView>::copy(Space& home, bool share) {
-    return new (home) ReEqDomInt<View,CtrlView>(home,share,*this);
+  ReEqDomInt<View,CtrlView,rm>::copy(Space& home, bool share) {
+    return new (home) ReEqDomInt<View,CtrlView,rm>(home,share,*this);
   }
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqDomInt<View,CtrlView>::propagate(Space& home, const ModEventDelta&) {
+  ReEqDomInt<View,CtrlView,rm>::propagate(Space& home, const ModEventDelta&) {
     if (b.one()) {
-      GECODE_ME_CHECK(x0.eq(home,c));
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(x0.eq(home,c));
     } else if (b.zero()) {
-      GECODE_ME_CHECK(x0.nq(home,c));
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(x0.nq(home,c));
     } else {
       switch (rtest_eq_dom(x0,c)) {
       case RT_TRUE:
-        GECODE_ME_CHECK(b.one_none(home)); break;
+        if (rm != RM_IMP)
+          GECODE_ME_CHECK(b.one_none(home));
+        break;
       case RT_FALSE:
-        GECODE_ME_CHECK(b.zero_none(home)); break;
+        if (rm != RM_PMI)
+          GECODE_ME_CHECK(b.zero_none(home));
+        break;
       case RT_MAYBE:
         return ES_FIX;
       default: GECODE_NEVER;
@@ -612,25 +655,29 @@ namespace Gecode { namespace Int { namespace Rel {
    *
    */
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqBndInt<View,CtrlView>::ReEqBndInt
+  ReEqBndInt<View,CtrlView,rm>::ReEqBndInt
   (Home home, View x, int c0, CtrlView b)
     : ReUnaryPropagator<View,PC_INT_BND,CtrlView>(home,x,b), c(c0) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqBndInt<View,CtrlView>::post(Home home, View x, int c, CtrlView b) {
+  ReEqBndInt<View,CtrlView,rm>::post(Home home, View x, int c, CtrlView b) {
     if (b.one()) {
-      GECODE_ME_CHECK(x.eq(home,c));
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(x.eq(home,c));
     } else if (b.zero()) {
-      GECODE_ME_CHECK(x.nq(home,c));
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(x.nq(home,c));
     } else if (x.assigned()) {
       assert(b.none());
       if (x.val() == c) {
-        GECODE_ME_CHECK(b.one_none(home));
+        if (rm != RM_IMP)
+          GECODE_ME_CHECK(b.one_none(home));
       } else {
-        GECODE_ME_CHECK(b.zero_none(home));
+        if (rm != RM_PMI)
+          GECODE_ME_CHECK(b.zero_none(home));
       }
     } else {
       (void) new (home) ReEqBndInt(home,x,c,b);
@@ -639,30 +686,36 @@ namespace Gecode { namespace Int { namespace Rel {
   }
 
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   forceinline
-  ReEqBndInt<View,CtrlView>::ReEqBndInt(Space& home, bool share, ReEqBndInt& p)
+  ReEqBndInt<View,CtrlView,rm>::ReEqBndInt(Space& home, bool share, ReEqBndInt& p)
     : ReUnaryPropagator<View,PC_INT_BND,CtrlView>(home,share,p), c(p.c) {}
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   Actor*
-  ReEqBndInt<View,CtrlView>::copy(Space& home, bool share) {
-    return new (home) ReEqBndInt<View,CtrlView>(home,share,*this);
+  ReEqBndInt<View,CtrlView,rm>::copy(Space& home, bool share) {
+    return new (home) ReEqBndInt<View,CtrlView,rm>(home,share,*this);
   }
 
-  template<class View, class CtrlView>
+  template<class View, class CtrlView, ReifyMode rm>
   ExecStatus
-  ReEqBndInt<View,CtrlView>::propagate(Space& home, const ModEventDelta&) {
+  ReEqBndInt<View,CtrlView,rm>::propagate(Space& home, const ModEventDelta&) {
     if (b.one()) {
-      GECODE_ME_CHECK(x0.eq(home,c));
+      if (rm != RM_PMI)
+        GECODE_ME_CHECK(x0.eq(home,c));
     } else if (b.zero()) {
-      GECODE_ME_CHECK(x0.nq(home,c));
+      if (rm != RM_IMP)
+        GECODE_ME_CHECK(x0.nq(home,c));
     } else {
       switch (rtest_eq_bnd(x0,c)) {
       case RT_TRUE:
-        GECODE_ME_CHECK(b.one_none(home)); break;
+        if (rm != RM_IMP)
+          GECODE_ME_CHECK(b.one_none(home)); 
+        break;
       case RT_FALSE:
-        GECODE_ME_CHECK(b.zero_none(home)); break;
+        if (rm != RM_PMI)
+          GECODE_ME_CHECK(b.zero_none(home)); 
+        break;
       case RT_MAYBE:
         return ES_FIX;
       default: GECODE_NEVER;
