@@ -7,8 +7,8 @@
  *     Christian Schulte, 2004
  *
  *  Last modified:
- *     $Date: 2012-09-07 17:42:21 +0200 (Fri, 07 Sep 2012) $ by $Author: schulte $
- *     $Revision: 13069 $
+ *     $Date: 2013-07-11 19:23:04 +0200 (Thu, 11 Jul 2013) $ by $Author: schulte $
+ *     $Revision: 13866 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -276,7 +276,8 @@ namespace Gecode { namespace Iter { namespace Ranges {
 
 
   forceinline
-  NaryUnion::NaryUnion(void) {}
+  NaryUnion::NaryUnion(void) 
+    : f(NULL) {}
 
   template<class I>
   forceinline void
