@@ -14,8 +14,8 @@
  *     Guido Tack, 2004
  *
  *  Last modified:
- *     $Date: 2013-02-20 18:27:38 +0100 (Wed, 20 Feb 2013) $ by $Author: schulte $
- *     $Revision: 13347 $
+ *     $Date: 2013-07-11 12:30:18 +0200 (Thu, 11 Jul 2013) $ by $Author: schulte $
+ *     $Revision: 13840 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -164,8 +164,6 @@ namespace Gecode {
   public:
     /// Allocate memory of size \a s
     void* alloc(SharedMemory* sm, size_t s);
-    /// Return how much memory has been allocated
-    size_t allocated(void) const;
     /// Get the memory area for subscriptions
     void* subscriptions(void) const;
 
@@ -326,11 +324,6 @@ namespace Gecode {
    *
    */
 
-  forceinline size_t
-  MemoryManager::allocated(void) const {
-    return requested;
-  }
-
   forceinline void*
   MemoryManager::alloc(SharedMemory* sm, size_t sz) {
     assert(sz > 0);
@@ -472,6 +465,13 @@ namespace Gecode {
   forceinline void
   MemoryManager::fl_dispose(FreeList* f, FreeList* l) {
     size_t i = sz2i(s);
+#ifdef GECODE_AUDIT
+    FreeList* cur = f;
+    while (cur != l) {
+      assert(cur->next());
+      cur = cur->next();
+    }
+#endif
     l->next(fl[i]); fl[i] = f;
   }
 
