@@ -7,8 +7,8 @@
  *     Christian Schulte, 2005
  *
  *  Last modified:
- *     $Date: 2010-10-11 21:10:44 +1100 (Mon, 11 Oct 2010) $ by $Author: tack $
- *     $Revision: 11514 $
+ *     $Date: 2013-02-18 21:53:58 +0100 (Mon, 18 Feb 2013) $ by $Author: schulte $
+ *     $Revision: 13310 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -52,8 +52,8 @@ namespace Gecode {
     //@{
     /// Return degree (number of subscribed propagators and advisors)
     unsigned int degree(void) const;
-    /// Return accumulated failure count (plus degree)
-    double afc(void) const;
+    /// Return accumulated failure count
+    double afc(const Space& home) const;
     /// Return whether this view is derived from a VarImpView
     static bool varderived(void);
     /// Return dummy variable implementation of view
@@ -136,8 +136,8 @@ namespace Gecode {
     VarImpType* varimp(void) const;
     /// Return degree (number of subscribed propagators and advisors)
     unsigned int degree(void) const;
-    /// Return accumulated failure count (plus degree)
-    double afc(void) const;
+    /// Return accumulated failure count
+    double afc(const Space& home) const;
     //@}
 
     /// \name Domain tests
@@ -229,8 +229,8 @@ namespace Gecode {
     View base(void) const;
     /// Return degree (number of subscribed propagators)
     unsigned int degree(void) const;
-    /// Return accumulated failure count (plus degree)
-    double afc(void) const;
+    /// Return accumulated failure count
+    double afc(const Space& home) const;
     //@}
 
     /// \name Domain tests
@@ -348,7 +348,7 @@ namespace Gecode {
   }
   template<class View>
   forceinline double
-  ConstView<View>::afc(void) const {
+  ConstView<View>::afc(const Space&) const {
     return 0.0;
   }
   template<class View>
@@ -440,8 +440,8 @@ namespace Gecode {
   }
   template<class Var>
   forceinline double
-  VarImpView<Var>::afc(void) const {
-    return x->afc();
+  VarImpView<Var>::afc(const Space& home) const {
+    return x->afc(home);
   }
   template<class Var>
   forceinline bool
@@ -534,8 +534,8 @@ namespace Gecode {
   }
   template<class View>
   forceinline double
-  DerivedView<View>::afc(void) const {
-    return x.afc();
+  DerivedView<View>::afc(const Space& home) const {
+    return x.afc(home);
   }
   template<class View>
   forceinline bool
