@@ -7,8 +7,8 @@
  *     Christian Schulte, 2003
  *
  *  Last modified:
- *     $Date: 2013-02-20 17:42:40 +0100 (Wed, 20 Feb 2013) $ by $Author: schulte $
- *     $Revision: 13341 $
+ *     $Date: 2013-07-11 12:30:18 +0200 (Thu, 11 Jul 2013) $ by $Author: schulte $
+ *     $Revision: 13840 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -39,13 +39,13 @@ namespace Gecode {
 
   namespace Search {
     /// Create depth-first engine
-    GECODE_SEARCH_EXPORT Engine* dfs(Space* s, size_t sz, const Options& o);
+    GECODE_SEARCH_EXPORT Engine* dfs(Space* s, const Options& o);
   }
 
   template<class T>
   forceinline
   DFS<T>::DFS(T* s, const Search::Options& o)
-    : EngineBase(Search::dfs(s,sizeof(T),o)) {}
+    : EngineBase(Search::dfs(s,o)) {}
 
   template<class T>
   forceinline T*
@@ -65,6 +65,12 @@ namespace Gecode {
     return e->stopped();
   }
 
+  template<class T>
+  forceinline NoGoods&
+  DFS<T>::nogoods(void) {
+    return e->nogoods();
+  }
+
 
   template<class T>
   forceinline T*
