@@ -7,8 +7,8 @@
  *     Christian Schulte, 2009
  *
  *  Last modified:
- *     $Date: 2013-03-07 20:56:21 +0100 (Thu, 07 Mar 2013) $ by $Author: schulte $
- *     $Revision: 13463 $
+ *     $Date: 2013-07-12 18:20:11 +0200 (Fri, 12 Jul 2013) $ by $Author: schulte $
+ *     $Revision: 13877 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -95,7 +95,7 @@ namespace Gecode { namespace Search { namespace Parallel {
             find();
           } else if (cur != NULL) {
             start();
-            if (stop(engine().opt(),path.size())) {
+            if (stop(engine().opt())) {
               // Report stop
               m.release();
               engine().stop();
@@ -112,7 +112,6 @@ namespace Gecode { namespace Search { namespace Parallel {
                 fail++;
                 delete cur;
                 cur = NULL;
-                Worker::current(NULL);
                 m.release();
                 break;
               case SS_SOLVED:
@@ -122,7 +121,6 @@ namespace Gecode { namespace Search { namespace Parallel {
                   Space* s = cur->clone(false);
                   delete cur;
                   cur = NULL;
-                  Worker::current(NULL);
                   m.release();
                   engine().solution(s);
                 }
@@ -138,7 +136,6 @@ namespace Gecode { namespace Search { namespace Parallel {
                     d++;
                   }
                   const Choice* ch = path.push(*this,cur,c);
-                  Worker::push(c,ch);
                   cur->commit(*ch,0);
                   m.release();
                 }
@@ -147,12 +144,12 @@ namespace Gecode { namespace Search { namespace Parallel {
                 GECODE_NEVER;
               }
             }
-          } else if (path.next(*this)) {
+          } else if (path.next()) {
             cur = path.recompute(d,engine().opt().a_d,*this,best,mark);
-            Worker::current(cur);
             m.release();
           } else {
             idle = true;
+            path.ngdl(0);
             m.release();
             // Report that worker is idle
             engine().idle();
@@ -183,8 +180,8 @@ namespace Gecode { namespace Search { namespace Parallel {
     best = NULL;
     n_busy = workers();
     for (unsigned int i=1; i<workers(); i++)
-      worker(i)->reset(NULL);
-    worker(0)->reset(s);
+      worker(i)->reset(NULL,0);
+    worker(0)->reset(s,opt().nogoods_limit);
     // Block workers again to ensure invariant
     block();
     // Release reset lock
@@ -194,6 +191,29 @@ namespace Gecode { namespace Search { namespace Parallel {
   }
 
 
+  /*
+   * Create no-goods
+   *
+   */
+  NoGoods&
+  BAB::nogoods(void) {
+    NoGoods* ng;
+    // Grab wait lock for reset
+    m_wait_reset.acquire();
+    // Release workers for reset
+    release(C_RESET);
+    // Wait for reset cycle started
+    e_reset_ack_start.wait();
+    ng = &worker(0)->nogoods();
+    // Block workers again to ensure invariant
+    block();
+    // Release reset lock
+    m_wait_reset.release();
+    // Wait for reset cycle stopped
+    e_reset_ack_stop.wait();
+    return *ng;
+  }
+
   /*
    * Termination and deletion
    */
