@@ -7,8 +7,8 @@
  *     Guido Tack, 2005
  *
  *  Last modified:
- *     $Date: 2011-08-25 00:34:16 +1000 (Thu, 25 Aug 2011) $ by $Author: tack $
- *     $Revision: 12346 $
+ *     $Date: 2012-10-19 05:58:26 +0200 (Fri, 19 Oct 2012) $ by $Author: tack $
+ *     $Revision: 13156 $
  *
  *  This file is part of Gecode, the generic constraint
  *  development environment:
@@ -122,11 +122,11 @@ namespace Test { namespace Set {
           Gecode::rel(home, x[0], srt, x[0]);
       }
       /// Post reified constraint on \a x for \a b
-      void post(Space& home, SetVarArray& x, IntVarArray&, BoolVar b) {
+      void post(Space& home, SetVarArray& x, IntVarArray&, Reify r) {
         if (!shared)
-          Gecode::rel(home, x[0], srt, x[1], b);
+          Gecode::rel(home, x[0], srt, x[1], r);
         else
-          Gecode::rel(home, x[0], srt, x[0], b);
+          Gecode::rel(home, x[0], srt, x[0], r);
       }
     };
     RelBin _relbin_eq(Gecode::SRT_EQ,false);
